__title__ = 'tld'
__version__ = '0.7.7'
__build__ = 0x000011
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('BaseTLDSource',)


class BaseTLDSource(object):
    """Base TLD source."""

    uid = None
